@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014-2017 -- leonerd@leonerd.org.uk
 
 package Event::Distributor::Query;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Event::Distributor::_Event );
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Future;
 
@@ -20,8 +20,9 @@ C<Event::Distributor::Query> - an event that collects a result
 =head1 DESCRIPTION
 
 This subclass of L<Event::Distributor::_Event> invokes each of its subscribers
-in turn, yielding either the (first) successful result, or a failure if they
-all fail.
+in turn, yielding either the (first) successful and non-empty result, or a
+failure if they all fail. Yields a (successful) empty result if there are no
+subscribers.
 
 =cut
 
@@ -34,16 +35,29 @@ sub fire
    my @f;
 
    foreach my $sub ( $self->subscribers ) {
-      my $f = $sub->( $dist, @args );
+      my $f = $sub->( $dist, @args )->then_with_f( sub {
+         my $f = shift;
+         return $f if @_;
+         die "No result\n";
+      });
+
       push @f, $f;
 
       last if $f->is_ready and !$f->failure;
    }
 
+   return Future->done if !@f;
+
    return Future->needs_any( @f )->then( sub {
       my @results = @_;
       # TODO: conversions?
       Future->done( @results );
+   })->else_with_f( sub {
+      my $f = shift;
+      my @other_fails = grep { $_->failure ne "No result\n" } $f->failed_futures;
+
+      return $other_fails[0] if @other_fails;
+      Future->done();
    });
 }
 
